@@ -4,7 +4,7 @@
 ## Copyright (C) 2015 Pavol Bauer
 ## Copyright (C) 2017 -- 2019 Robin Eriksson
 ## Copyright (C) 2015 -- 2019 Stefan Engblom
-## Copyright (C) 2015 -- 2019 Stefan Widgren
+## Copyright (C) 2015 -- 2020 Stefan Widgren
 ##
 ## SimInf is free software: you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -208,7 +208,42 @@ C_code <- c(
     "#include <R_ext/Rdynload.h>",
     "#include \"SimInf.h\"",
     "",
-    "double trFun1(",
+    "/**",
+    " * Make sure the necessary macros are defined so that the",
+    " * compiler can replace them when compiling the model.",
+    " * 'SIMINF_MODEL_RUN' defines the function name of the function",
+    " * that will be called from R to run a trajectory of the model.",
+    " * 'SIMINF_R_INIT' is the name of the function that R will call",
+    " * when this model is loaded into R. 'SIMINF_FORCE_SYMBOLS'",
+    " * defines whether R allows the entry point for the run function",
+    " * to be searched for as a character string.",
+    " * If this file is compiled from SimInf (when calling run), the",
+    " * macros are defined by SimInf before calling 'R CMD SHLIB'.",
+    " * If this file is compiled as part of a package, then the",
+    " * definitions are set in the variable 'PKG_CPPFLAGS' in",
+    " * 'src/Makevars' and 'src/Makevars.in'.",
+    " */",
+    "#if !defined(SIMINF_MODEL_RUN)",
+    "#  error Definition for 'SIMINF_MODEL_RUN' is missing.",
+    "#endif",
+    "#if !defined(SIMINF_R_INIT)",
+    "#  error Definition for 'SIMINF_R_INIT' is missing.",
+    "#endif",
+    "#if !defined(SIMINF_FORCE_SYMBOLS)",
+    "#  error Definition for 'SIMINF_FORCE_SYMBOLS' is missing.",
+    "#endif",
+    "#define SIMINF_STR(name) #name",
+    "#define SIMINF_CALLDEF(name, n) {SIMINF_STR(name), (DL_FUNC) &name, n}",
+    "",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun1(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -218,7 +253,15 @@ C_code <- c(
     "    return gdata[0];",
     "}",
     "",
-    "double trFun2(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun2(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -228,7 +271,15 @@ C_code <- c(
     "    return gdata[1]*u[0];",
     "}",
     "",
-    "double trFun3(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun3(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -238,7 +289,15 @@ C_code <- c(
     "    return gdata[2]*u[0]*u[1];",
     "}",
     "",
-    "double trFun4(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun4(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -248,7 +307,24 @@ C_code <- c(
     "    return gdata[3]*u[1];",
     "}",
     "",
-    "int ptsFun(",
+    "/**",
+    " * Post time step function.",
+    " *",
+    " * @param v_new If a continuous state vector is used by a model,",
+    " *        this is the new continuous state vector in the node after",
+    " *        the post time step.",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The current continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector that is common to all nodes.",
+    " * @param node The node index. Note the node index is zero-based,",
+    " *        i.e., the first node is 0.",
+    " * @param t Current time in the simulation.",
+    " * @return error code (<0), or 1 if node needs to update the",
+    " *         transition rates, or 0 when it doesn't need to update",
+    " *         the transition rates.",
+    " */",
+    "static int ptsFun(",
     "    double *v_new,",
     "    const int *u,",
     "    const double *v,",
@@ -260,13 +336,43 @@ C_code <- c(
     "    return 0;",
     "}",
     "",
-    "SEXP SimInf_model_run(SEXP model, SEXP threads, SEXP solver)",
+    "/**",
+    " * Run a trajectory of the model.",
+    " *",
+    " * @param model The model.",
+    " * @param solver The name of the numerical solver.",
+    " * @return A model with a trajectory attached to it.",
+    " */",
+    "static SEXP SIMINF_MODEL_RUN(SEXP model, SEXP solver)",
     "{",
     "    TRFun tr_fun[] = {&trFun1, &trFun2, &trFun3, &trFun4};",
     "    DL_FUNC SimInf_run = R_GetCCallable(\"SimInf\", \"SimInf_run\");",
-    "    return SimInf_run(model, threads, solver, tr_fun, &ptsFun);",
+    "    return SimInf_run(model, solver, tr_fun, &ptsFun);",
     "}",
-    "")
+    "",
+    "/**",
+    " * A NULL-terminated array of routines to register for the .Call",
+    " * interface, see section '5.4 Registering native routines' in",
+    " * the 'Writing R Extensions' manual.",
+    " */",
+    "static const R_CallMethodDef callMethods[] =",
+    "{",
+    "    SIMINF_CALLDEF(SIMINF_MODEL_RUN, 2),",
+    "    {NULL, NULL, 0}",
+    "};",
+    "",
+    "/**",
+    " * This routine will be invoked when R loads the shared object/DLL,",
+    " * see section '5.4 Registering native routines' in the",
+    " * 'Writing R Extensions' manual.",
+    " */",
+    "void SIMINF_R_INIT(DllInfo *info)",
+    "{",
+    "    R_registerRoutines(info, NULL, callMethods, NULL, NULL);",
+    "    R_useDynamicSymbols(info, FALSE);",
+    "    R_forceSymbols(info, SIMINF_FORCE_SYMBOLS);",
+    "}")
+
 stopifnot(identical(m@C_code[-1], C_code)) ## Skip first line that contains time
 stopifnot(identical(m@C_code, C_code(m)))
 
@@ -283,7 +389,42 @@ C_code <- c(
     "#include <R_ext/Rdynload.h>",
     "#include \"SimInf.h\"",
     "",
-    "double trFun1(",
+    "/**",
+    " * Make sure the necessary macros are defined so that the",
+    " * compiler can replace them when compiling the model.",
+    " * 'SIMINF_MODEL_RUN' defines the function name of the function",
+    " * that will be called from R to run a trajectory of the model.",
+    " * 'SIMINF_R_INIT' is the name of the function that R will call",
+    " * when this model is loaded into R. 'SIMINF_FORCE_SYMBOLS'",
+    " * defines whether R allows the entry point for the run function",
+    " * to be searched for as a character string.",
+    " * If this file is compiled from SimInf (when calling run), the",
+    " * macros are defined by SimInf before calling 'R CMD SHLIB'.",
+    " * If this file is compiled as part of a package, then the",
+    " * definitions are set in the variable 'PKG_CPPFLAGS' in",
+    " * 'src/Makevars' and 'src/Makevars.in'.",
+    " */",
+    "#if !defined(SIMINF_MODEL_RUN)",
+    "#  error Definition for 'SIMINF_MODEL_RUN' is missing.",
+    "#endif",
+    "#if !defined(SIMINF_R_INIT)",
+    "#  error Definition for 'SIMINF_R_INIT' is missing.",
+    "#endif",
+    "#if !defined(SIMINF_FORCE_SYMBOLS)",
+    "#  error Definition for 'SIMINF_FORCE_SYMBOLS' is missing.",
+    "#endif",
+    "#define SIMINF_STR(name) #name",
+    "#define SIMINF_CALLDEF(name, n) {SIMINF_STR(name), (DL_FUNC) &name, n}",
+    "",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun1(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -293,7 +434,15 @@ C_code <- c(
     "    return gdata[0];",
     "}",
     "",
-    "double trFun2(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun2(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -303,7 +452,15 @@ C_code <- c(
     "    return gdata[1]*u[0];",
     "}",
     "",
-    "double trFun3(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun3(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -313,7 +470,15 @@ C_code <- c(
     "    return gdata[2]*u[0]*u[1];",
     "}",
     "",
-    "double trFun4(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun4(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -323,7 +488,24 @@ C_code <- c(
     "    return ldata[0]*u[1];",
     "}",
     "",
-    "int ptsFun(",
+    "/**",
+    " * Post time step function.",
+    " *",
+    " * @param v_new If a continuous state vector is used by a model,",
+    " *        this is the new continuous state vector in the node after",
+    " *        the post time step.",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The current continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector that is common to all nodes.",
+    " * @param node The node index. Note the node index is zero-based,",
+    " *        i.e., the first node is 0.",
+    " * @param t Current time in the simulation.",
+    " * @return error code (<0), or 1 if node needs to update the",
+    " *         transition rates, or 0 when it doesn't need to update",
+    " *         the transition rates.",
+    " */",
+    "static int ptsFun(",
     "    double *v_new,",
     "    const int *u,",
     "    const double *v,",
@@ -335,13 +517,43 @@ C_code <- c(
     "    return 0;",
     "}",
     "",
-    "SEXP SimInf_model_run(SEXP model, SEXP threads, SEXP solver)",
+    "/**",
+    " * Run a trajectory of the model.",
+    " *",
+    " * @param model The model.",
+    " * @param solver The name of the numerical solver.",
+    " * @return A model with a trajectory attached to it.",
+    " */",
+    "static SEXP SIMINF_MODEL_RUN(SEXP model, SEXP solver)",
     "{",
     "    TRFun tr_fun[] = {&trFun1, &trFun2, &trFun3, &trFun4};",
     "    DL_FUNC SimInf_run = R_GetCCallable(\"SimInf\", \"SimInf_run\");",
-    "    return SimInf_run(model, threads, solver, tr_fun, &ptsFun);",
+    "    return SimInf_run(model, solver, tr_fun, &ptsFun);",
     "}",
-    "")
+    "",
+    "/**",
+    " * A NULL-terminated array of routines to register for the .Call",
+    " * interface, see section '5.4 Registering native routines' in",
+    " * the 'Writing R Extensions' manual.",
+    " */",
+    "static const R_CallMethodDef callMethods[] =",
+    "{",
+    "    SIMINF_CALLDEF(SIMINF_MODEL_RUN, 2),",
+    "    {NULL, NULL, 0}",
+    "};",
+    "",
+    "/**",
+    " * This routine will be invoked when R loads the shared object/DLL,",
+    " * see section '5.4 Registering native routines' in the",
+    " * 'Writing R Extensions' manual.",
+    " */",
+    "void SIMINF_R_INIT(DllInfo *info)",
+    "{",
+    "    R_registerRoutines(info, NULL, callMethods, NULL, NULL);",
+    "    R_useDynamicSymbols(info, FALSE);",
+    "    R_forceSymbols(info, SIMINF_FORCE_SYMBOLS);",
+    "}")
+
 stopifnot(identical(m@C_code[-1], C_code)) ## Skip first line that contains time
 
 stopifnot(identical(SimInf:::tokens("beta*S*I/(S+I+R)"),
@@ -368,7 +580,42 @@ C_code <- c(
     "#include <R_ext/Rdynload.h>",
     "#include \"SimInf.h\"",
     "",
-    "double trFun1(",
+    "/**",
+    " * Make sure the necessary macros are defined so that the",
+    " * compiler can replace them when compiling the model.",
+    " * 'SIMINF_MODEL_RUN' defines the function name of the function",
+    " * that will be called from R to run a trajectory of the model.",
+    " * 'SIMINF_R_INIT' is the name of the function that R will call",
+    " * when this model is loaded into R. 'SIMINF_FORCE_SYMBOLS'",
+    " * defines whether R allows the entry point for the run function",
+    " * to be searched for as a character string.",
+    " * If this file is compiled from SimInf (when calling run), the",
+    " * macros are defined by SimInf before calling 'R CMD SHLIB'.",
+    " * If this file is compiled as part of a package, then the",
+    " * definitions are set in the variable 'PKG_CPPFLAGS' in",
+    " * 'src/Makevars' and 'src/Makevars.in'.",
+    " */",
+    "#if !defined(SIMINF_MODEL_RUN)",
+    "#  error Definition for 'SIMINF_MODEL_RUN' is missing.",
+    "#endif",
+    "#if !defined(SIMINF_R_INIT)",
+    "#  error Definition for 'SIMINF_R_INIT' is missing.",
+    "#endif",
+    "#if !defined(SIMINF_FORCE_SYMBOLS)",
+    "#  error Definition for 'SIMINF_FORCE_SYMBOLS' is missing.",
+    "#endif",
+    "#define SIMINF_STR(name) #name",
+    "#define SIMINF_CALLDEF(name, n) {SIMINF_STR(name), (DL_FUNC) &name, n}",
+    "",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun1(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -378,7 +625,15 @@ C_code <- c(
     "    return gdata[0]*u[0]*u[1]/(u[0]+u[1]+u[2]);",
     "}",
     "",
-    "double trFun2(",
+    "/**",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector.",
+    " * @param t Current time.",
+    " * @return propensity.",
+    " */",
+    "static double trFun2(",
     "    const int *u,",
     "    const double *v,",
     "    const double *ldata,",
@@ -388,7 +643,24 @@ C_code <- c(
     "    return gdata[1]*u[1];",
     "}",
     "",
-    "int ptsFun(",
+    "/**",
+    " * Post time step function.",
+    " *",
+    " * @param v_new If a continuous state vector is used by a model,",
+    " *        this is the new continuous state vector in the node after",
+    " *        the post time step.",
+    " * @param u The compartment state vector in the node.",
+    " * @param v The current continuous state vector in the node.",
+    " * @param ldata The local data vector in the node.",
+    " * @param gdata The global data vector that is common to all nodes.",
+    " * @param node The node index. Note the node index is zero-based,",
+    " *        i.e., the first node is 0.",
+    " * @param t Current time in the simulation.",
+    " * @return error code (<0), or 1 if node needs to update the",
+    " *         transition rates, or 0 when it doesn't need to update",
+    " *         the transition rates.",
+    " */",
+    "static int ptsFun(",
     "    double *v_new,",
     "    const int *u,",
     "    const double *v,",
@@ -400,13 +672,42 @@ C_code <- c(
     "    return 0;",
     "}",
     "",
-    "SEXP SimInf_model_run(SEXP model, SEXP threads, SEXP solver)",
+    "/**",
+    " * Run a trajectory of the model.",
+    " *",
+    " * @param model The model.",
+    " * @param solver The name of the numerical solver.",
+    " * @return A model with a trajectory attached to it.",
+    " */",
+    "static SEXP SIMINF_MODEL_RUN(SEXP model, SEXP solver)",
     "{",
     "    TRFun tr_fun[] = {&trFun1, &trFun2};",
     "    DL_FUNC SimInf_run = R_GetCCallable(\"SimInf\", \"SimInf_run\");",
-    "    return SimInf_run(model, threads, solver, tr_fun, &ptsFun);",
+    "    return SimInf_run(model, solver, tr_fun, &ptsFun);",
     "}",
-    "")
+    "",
+    "/**",
+    " * A NULL-terminated array of routines to register for the .Call",
+    " * interface, see section '5.4 Registering native routines' in",
+    " * the 'Writing R Extensions' manual.",
+    " */",
+    "static const R_CallMethodDef callMethods[] =",
+    "{",
+    "    SIMINF_CALLDEF(SIMINF_MODEL_RUN, 2),",
+    "    {NULL, NULL, 0}",
+    "};",
+    "",
+    "/**",
+    " * This routine will be invoked when R loads the shared object/DLL,",
+    " * see section '5.4 Registering native routines' in the",
+    " * 'Writing R Extensions' manual.",
+    " */",
+    "void SIMINF_R_INIT(DllInfo *info)",
+    "{",
+    "    R_registerRoutines(info, NULL, callMethods, NULL, NULL);",
+    "    R_useDynamicSymbols(info, FALSE);",
+    "    R_forceSymbols(info, SIMINF_FORCE_SYMBOLS);",
+    "}")
 
 ## Skip first line that contains time
 stopifnot(identical(model@C_code[-1], C_code))
@@ -523,7 +824,7 @@ m  <- mparse(transitions = ".S.S -> 1.2*.S.S -> @",
              compartments = c(".S.S"),
              u0 = data.frame(.S.S = 100),
              tspan = 1:100)
-stopifnot(identical(m@C_code[13], "    return 1.2*u[0];"))
+stopifnot(identical(m@C_code[48], "    return 1.2*u[0];"))
 
 ## Check mparse with a propensity that contains '->' to handle a case
 ## where a pointer is used in the propensity.
@@ -531,20 +832,7 @@ m  <- mparse(transitions = "S -> a->data[2]*1.2*S -> @",
              compartments = c("S"),
              u0 = data.frame(S = 100),
              tspan = 1:100)
-stopifnot(identical(m@C_code[13], "    return a->data[2]*1.2*u[0];"))
-
-run_model <- function(model) {
-    ## The environmental variable "R_TEST" must be unset inside "R CMD
-    ## check" in order to successfully change the working directory to
-    ## a tempdir and then run "R CMD SHLIB".
-    R_TESTS <- Sys.getenv("R_TESTS", unset = NA)
-    if (!is.na(R_TESTS)) {
-        Sys.unsetenv("R_TESTS")
-        on.exit(Sys.setenv(R_TESTS = R_TESTS), add = TRUE)
-    }
-
-    run(model)
-}
+stopifnot(identical(m@C_code[48], "    return a->data[2]*1.2*u[0];"))
 
 model <- mparse(transitions = c("S -> beta*S*I/(S+I+R) -> I",
                                 "I -> gamma*I -> R"),
@@ -553,8 +841,15 @@ model <- mparse(transitions = c("S -> beta*S*I/(S+I+R) -> I",
                 u0 = data.frame(S = 100:105, I = 1:6, R = rep(0, 6)),
                 tspan = 1:10)
 
+## Test that the environmental variable PKG_CPPFLAGS is restored after
+## compiling C code for a model. First set it to a known value.
+Sys.setenv(PKG_CPPFLAGS = "test")
+
 set.seed(22)
-result <- run_model(model)
+result <- run(model)
+
+## Then test that PKG_CPPFLAGS is restored.
+stopifnot(identical(Sys.getenv("PKG_CPPFLAGS"), "test"))
 
 U_exp <- data.frame(
     node = c(1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L, 5L, 6L, 1L, 2L, 3L, 4L,
@@ -584,6 +879,12 @@ U_exp <- data.frame(
 
 stopifnot(identical(trajectory(result), U_exp))
 
+## Remove the C code and check that an error is raised when calling
+## 'run'.
+model@C_code <- character(0)
+res <- assertError(run(model))
+check_error(res, "The model must contain C code.")
+
 ## Check that mparse fails with invalid usage of the empty set '@'.
 res <- assertError(
     mparse(transitions = c("S -> beta*S*I/(S+I+R) -> I",
